@@ -21,8 +21,7 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 */
 
-#ifndef SRC_PLAYER_H_CN979RE9
-#define SRC_PLAYER_H_CN979RE9
+#pragma once
 
 #include "config.h"
 
@@ -63,6 +62,7 @@ typedef struct {
 	AVFilterGraph *fgraph;
 	AVFormatContext *fctx;
 	AVStream *st;
+	AVCodecContext *cctx;
 	AVFilterContext *fbufsink, *fabuf;
 	int streamIdx;
 	int64_t lastTimestamp;
@@ -87,4 +87,3 @@ void BarPlayerSetVolume (player_t * const player);
 void BarPlayerInit ();
 void BarPlayerDestroy ();
 
-#endif /* SRC_PLAYER_H_CN979RE9 */
